@@ -115,92 +115,87 @@ class Decorators:
     # parameter helpers ------------------------------------------------
     RX_EMAIL = re.compile (r"[a-z0-9][-.a-z0-9]*@[-a-z0-9]+\.[-.a-z0-9]{2,}[a-z]$", re.I)
     RX_UUID = re.compile (r"[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}", re.I)
-    def _validate_param (self, params, required, ints, floats, emails, uuids, nones, lists, strings, booleans, dicts, **kargs):
+    RX_UNSAFE = re.compile (r"(<script[\s>]|['\"=]\s*javascript:|\son[:a-z]+\s*=\s*|&#x[0-9a-f]{2};|┻━┻)", re.I)
+    PARAM_CUSTOM_TYPES = {'required', 'leastone', 'emails', 'uuids'}
+    PARAM_TYPES = {'ints', 'floats', 'lists', 'strings', 'dicts', 'booleans', 'ranges', 'safes', 'notags'}
+    def _validate_param (self, params, **kargs):
         params = params or {}
+        for k in list (kargs.keys ()):
+            if k in self.PARAM_CUSTOM_TYPES:
+                fields = kargs.pop (k)
+                if k == 'required':
+                    for each in fields:
+                        if not params.get (each):
+                            return 'parameter {} required'.format (each)
 
-        if required:
-            for each in required:
-                if not params.get (each):
-                    return 'parameter {} required'.format (each)
+                elif k == 'leastone':
+                    vals = []
+                    for fd in fields:
+                        vals.append (params.get (fd) is not None and 1 or 0)
+                    if sum (vals) == 0:
+                        return 'at least one parameter of {} are required'.format (', '.join (leastone))
 
-        if ints:
-            for each in ints:
-                try: val = params [each]
-                except KeyError: continue
-                if not val: val = 0
-                try:
-                    params [each] = int (val)
-                except ValueError:
-                    return 'parameter {} should be integer'.format (each)
-        if floats:
-            for each in floats:
-                try: val = params [each]
-                except KeyError: continue
-                if not val: val = 0.
-                try:
-                    params [each] = float (val)
-                except ValueError:
-                    return 'parameter {} should be float'.format (each)
+                elif k == 'emails':
+                    for fd in fields:
+                        kargs [fd] = self.RX_EMAIL
 
-        if lists:
-            for each in lists:
-                try: val = params [each]
-                except KeyError: continue
-                if val is None: val = []
-                if not isinstance (val, (str, list, tuple)):
-                    return 'parameter {} should be list or comma/bar delimtered string'.format (each)
-                if isinstance (val, str):
-                    if val.find ("|") != -1:
-                        val = val.split ("|")
-                    else:
-                        val = val.split (",")
-                params [each] = val
+                elif k == 'uuids':
+                    for fd in fields:
+                        kargs [fd] = self.RX_UUID
 
-        if strings:
-            for each in strings:
-                try: val = params [each]
-                except KeyError: continue
-                if val is None: val = ''
-                if isinstance (val, str):
-                    params [each] = val
-                else:
-                    return 'parameter {} should be string'.format (each)
+            elif k in self.PARAM_TYPES:
+                types = kargs.pop (k)
+                for each in types:
+                    try:
+                        val = params [each]
+                    except KeyError:
+                        continue
 
-        if dicts:
-            for each in dicts:
-                try: val = params [each]
-                except KeyError: continue
-                if val is None: val = {}
-                if isinstance (val, dict):
-                    params [each] = val
-                else:
-                    return 'parameter {} should be object'.format (each)
+                    if val is None:
+                        continue
 
-        if booleans:
-            for each in booleans:
-                try: val = params [each]
-                except KeyError: continue
-                if val in ('yes', 'true'): val = True
-                elif val in ('no', 'false'): val = False
-                if val in (True, False):
-                    params [each] = val
-                else:
-                    return 'parameter {} should be boolean'.format (each)
+                    try:
+                        if k == 'ints':
+                            val = int (val)
+                        elif k == 'floats':
+                            val = float (val)
+                        elif k == 'lists':
+                            if not isinstance (val, (str, list, tuple)):
+                                return 'parameter {} should be a list or bar delimtered string'.format (each)
+                            if isinstance (val, str):
+                                val = val.split ("|")
+                        elif k == 'strings':
+                            if not isinstance (val, str):
+                                raise ValueError
+                        elif k == 'dicts':
+                            if not isinstance (val, dict):
+                                raise ValueError
+                        elif k == 'booleans':
+                            if val in ('yes', 'true', 'y', 't'): val = True
+                            elif val in ('no', 'false', 'n', 'f'): val = False
+                            if val not in (True, False):
+                                return 'parameter {} should be a boolean (one of yes, no, y, n, t, f, true or false)'.format (each)
+                        elif k == 'ranges':
+                            a, b = map (int, val.split ('~'))
+                            val = (a, b)
+                        elif k == 'safes':
+                            if self.RX_UNSAFE.search (val):
+                                return 'parameter {} is unsafe'.format (each)
+                        elif k == 'notags':
+                            val = val.replace ('<', '&lt;').replace ('>', '&gt;')
+                        params [each] = val
 
-        if emails:
-            for fd in emails:
-                kargs [fd] = self.RX_EMAIL
-        if uuids:
-            for fd in uuids:
-                kargs [fd] = self.RX_UUID
+                    except ValueError:
+                        return 'parameter {} should be {} type'.format (each, k [:-1])
 
         for fd_, cond in kargs.items ():
             ops = fd_.split ("__")
             fd = ops [0]
             val = params.get (fd)
-            if not val:
+            if val is None:
                 continue
-            assert len (ops) <= 3 and fd, "Invalid expression"
+            if not (len (ops) <= 3 and fd):
+                raise SyntaxError ("invalid require expression on {}".format (fd))
             if len (ops) == 1:
                 if hasattr (cond, "search"):
                     if not cond.search (val):
@@ -216,13 +211,24 @@ class Decorators:
 
             op = ops [-1]
             val = (isinstance (cond, (list, tuple)) and type (cond [0]) or type (cond)) (val)
-            if op in ("eq", "exact"):
+
+            # mostly comaptible with sqlphile operators -------------------
+            if op == "between":
+                if not (cond [0] <= val <= cond [1]):
+                    return 'parameter {} should be between {} ~ {}'.format (fd, cond [0], cond [1])
+            elif op == "in":
+                if val not in cond:
+                    return 'parameter {} should be one of {}'.format (fd, cond)
+            elif op == "notin":
+                if val in cond:
+                    return 'parameter {} should be not one of {}'.format (fd, cond)
+
+            elif op in ("eq", "exact"):
                 if val != cond:
                     return 'parameter {} should be {}'.format (fd, cond)
             elif op == "neq":
                 if val == cond:
                     return 'parameter {} should not be {}'.format (fd, cond)
-
             elif op == "lte":
                 if val > cond:
                     return 'parameter {} should less or equal than {}'.format (fd, cond)
@@ -235,15 +241,6 @@ class Decorators:
             elif op == "gt":
                 if val <= cond:
                     return 'parameter {} should greater than {}'.format (fd, cond)
-            elif op == "between":
-                if not (cond [0] <= val <= cond [1]):
-                    return 'parameter {} should be between {} ~ {}'.format (fd, cond [0], cond [1])
-            elif op == "in":
-                if val not in cond:
-                    return 'parameter {} should be one of {}'.format (fd, cond)
-            elif op == "notin":
-                if val in cond:
-                    return 'parameter {} should be not one of {}'.format (fd, cond)
 
             elif op == "startswith":
                 if not val.startswith (cond):
@@ -264,10 +261,14 @@ class Decorators:
                 if val.find (cond) != -1:
                     return 'parameter {} should be not contain {}'.format (fd, cond)
 
+            elif op == "regex":
+                if not re.compile (cond).search (val):
+                    return 'parameter {} should not be {}'.format (fd, cond)
             else:
                 raise ValueError ("Unknown operator: {}".format (op))
 
-        if nones:
+        if 'nones' in kargs: # must be None if not val
+            nones = kargs ['nones']
             for each in nones:
                 try:
                     if not params [each]:
@@ -278,14 +279,24 @@ class Decorators:
     def get_parameter_requirements (self, func_id):
         return self._parameter_caches.get (func_id, {})
 
-    def require (self, scope = 'ARGS', required = None, ints = None, floats = None, uuids = None, emails = None, nones = None, lists = None, strings = None, booleans = None, dicts = None, **kargs):
-        def decorator(f):
+    def require (self, scope = 'ARGS', required = None, **kargs):
+        # required, leastone, ints, floats, uuids, emails, nones, lists, strings, booleans, dicts, notags, safes, ranges, **kargs
+        def decorator (f):
             self.save_function_spec (f)
             func_id = self.get_func_id (f)
             if func_id not in self._parameter_caches:
                 self._parameter_caches [func_id] = {}
-            self._parameter_caches [func_id][scope] = (required, ints, floats, uuids)
-            @wraps(f)
+            if required:
+                kargs ['required'] = required
+
+            for k, v in kargs.items ():
+                if (k in self.PARAM_CUSTOM_TYPES or k in self.PARAM_TYPES) and isinstance (v, str):
+                    kargs [k] = v.split ()
+                else:
+                    kargs [k] = v
+            self._parameter_caches [func_id][scope] = kargs
+
+            @wraps (f)
             def wrapper (was, *args, **kwargs):
                 scope_ = scope
                 if scope in ("FORM", "JSON", "DATA"):
@@ -308,9 +319,9 @@ class Decorators:
                         return f (was, *args, **kwargs)
 
                 validatable_args = getattr (was.request, scope_)
-                more_info = self._validate_param (validatable_args, required, ints, floats, emails, uuids, nones, lists, strings, booleans, dicts, **kargs)
+                more_info = self._validate_param (validatable_args, **kargs)
                 if more_info:
-                    return was.response.adaptive_error ("400 Bad Request", 'missing or bad parameterin {}'.format (scope_), 40050, more_info)
+                    return was.response.adaptive_error ("400 Bad Request", 'missing or bad parameter in {}'.format (scope_), 40050, more_info)
 
                 # syncing args --------------------------------------
                 for k, v in validatable_args.items ():
