@@ -172,7 +172,10 @@ class WAS (wsgiappservice.WAS):
     @property
     def csrf_token (self):
         if self.CSRF_NAME not in self.cookie:
-            tok = hex (random.getrandbits (64)) [2:]
+            if self.app.debug:
+                tok = '3649c350861db268'
+            else:
+                tok = hex (random.getrandbits (64)) [2:]
             self.cookie [self.CSRF_NAME] = tok
         return self.cookie [self.CSRF_NAME]
 
